==> hyades.source_1.3.0_20040422_2239.zip: org/eclipse/hyades/trace/views/internal/ExecutionStatistic.java <==
/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.trace.views.internal;

import org.eclipse.hyades.models.trace.TRCClass;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCMethodInvocation;
import org.eclipse.hyades.models.trace.TRCObject;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.trace.ui.TraceViewerPage;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.ViewSelectionChangedEvent;
import org.eclipse.hyades.trace.views.adapter.internal.ExecutionStatisticPage;
import org.eclipse.hyades.trace.views.adapter.internal.TraceConstants;
import org.eclipse.hyades.trace.views.util.internal.ColumnData;
import org.eclipse.hyades.trace.views.util.internal.ITimeChangedListener;
import org.eclipse.hyades.trace.views.util.internal.PerftraceUtil;
import org.eclipse.hyades.trace.views.util.internal.StatisticTableColumnInfo;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TableTreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.custom.TableTreeItem;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

public class ExecutionStatistic extends StatisticView
	                              implements ITimeChangedListener

{
    protected String tmpString;
    
	private int _drawMode = TraceUIPlugin.getDefault().getPreferenceStore().getInt(TraceConstants.TIME_OPTION);
	
	/**
	 * 
	 *
	 */
	public class ExecutionStatisticFilter extends StatisticFilter {

		public ExecutionStatisticFilter() {
			super();
		}

		public boolean select(Viewer viewer, Object parent, Object element) {
			
			boolean flag = true;
			
			if(!(element instanceof TRCThread))
			  return true;
			  
			String compareText = "";
		   compareText = ((TRCThread)element).getName();
		   if(compareText.equals(""))
			 compareText = TraceUIPlugin.getString("DEFAULT_PACKAGE");
			
			if (_noPattern)
				return true;
			
			if (!_caseSensitive) {
				compareText = compareText.toLowerCase();
			}
			if (_exactMatch) {
				return compareText.compareTo(_prefix) == 0;
			}
			if (_prefix != "") {
				flag = compareText.startsWith(_prefix);
			}
			if (flag && _suffix != "") {
				flag = compareText.endsWith(_suffix);
			}
			if (flag) {
				for (int k = 0; k < _textList.size(); k++) {
					String str1 = (String) _textList.get(k);

					int index = compareText.lastIndexOf(str1);
					if (index == -1) {
						flag = false;
						break;
					}

					compareText = compareText.substring(index + str1.length());
				}
			}
			return flag;
		}
		
	}
	
	public class ExecutionStatisticSorter extends StatisticSorter {

		public ExecutionStatisticSorter() {
			super();
		}

		public int compare(Viewer viewer, Object e1, Object e2) {
			
			double d = 0;
			
			if(e1 instanceof TRCThread && e2 instanceof TRCThread)
			{
				if(_pos == 0)
				  return (int) _sortSequence * (int) PerftraceUtil.getThreadName((TRCThread)e1).compareToIgnoreCase(PerftraceUtil.getThreadName((TRCThread)e2));
			}
			
			if(e1 instanceof TRCFullMethodInvocation && e2 instanceof TRCFullMethodInvocation)
			{
				TRCFullMethodInvocation inv1 = (TRCFullMethodInvocation)e1;
				TRCFullMethodInvocation inv2 = (TRCFullMethodInvocation)e2;
				
				if(_pos == 0) //name
				{
				   return (int) _sortSequence * (int) ((TRCMethodInvocation)e1).getMethod().getName().compareToIgnoreCase(((TRCMethodInvocation)e2).getMethod().getName());
				}
				
				if(_pos == 1) //instances
				{					
				   return (int) _sortSequence * (int) PerftraceUtil.getClass(inv1).getName().compareToIgnoreCase(PerftraceUtil.getClass(inv2).getName());
				}
				
				if(_pos == 2)//base time
				{
					  d = inv1.getEntryTime() - inv2.getEntryTime();
					  
					  if(d < 0)
					     return  -1 * _sortSequence;
					  if(d > 0) 
					    return _sortSequence;
					    
					  return 0;    
				}

				if(_pos == 3) //cumulative time 
				{
					double start = inv1.getEntryTime();
					double end = inv1.getExitTime();
					if(end == 0)
					{
						end = inv1.getOwningObject().getProcess().getLastEventTime();
					}
					d = end-start;
					
					start = inv2.getEntryTime();
					end = inv2.getExitTime();
					if(end == 0)
					{
						end = inv2.getOwningObject().getProcess().getLastEventTime();
					}
					
					  d = d - (end -start);
					  
					  if(d < 0)
					     return  -1 * _sortSequence;
					  if(d > 0) 
					    return _sortSequence;
					    
					  return 0;    
				}						
			}
			
			return 0;
		}
		
	}
	
	/**
	 * 
	 */
	public class ExecutionStatisticContentProvider implements ITreeContentProvider {
		
		public void dispose() {
		}
		
		public Object getParent(Object element) {
			return null;
		}
		
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
		
		public Object[] getElements(Object inputElement)
		{
			return PerftraceUtil.getAllThreads(_page.getMOFObject());
		}
				
		public Object[] getChildren(Object element)
		{
			tmpList.clear();
			
			if(element instanceof TRCThread)
				return ((TRCThread)element).getInitialInvocations().toArray();
			
			TRCMethodInvocation inv = (TRCMethodInvocation)element;
			Object[] segments = inv.getInvokes().toArray();
			for(int idx=0; idx<segments.length; idx++)
			{
				tmpList.add(((TRCMethodInvocation)segments[idx]));
			}
			
			return tmpList.toArray(); 	
		}
	
		public boolean hasChildren(Object element) {
			if (element instanceof TRCThread)
				return true;

			return (((TRCMethodInvocation)element).getInvokes().size() > 0);
		}
	}

    /**
     * 
     */
	public class ExecutionStatisticLableProvider extends LabelProvider implements ITableLabelProvider
	{
		protected StatisticView _viewer;
		
		public ExecutionStatisticLableProvider(StatisticView viewer) {
			super();
			_viewer = viewer;
		}
		
		public Image getColumnImage(Object obj, int col) {
				return null;
		}

		public String getColumnText(Object obj, int col) {
			
            int pos = StatisticTableColumnInfo.getStatisticTableColumnInfo(_viewer.getTable().getColumn(col)).getColumnData().getInitalPos();			
			
			if(obj instanceof TRCThread)	
			{
				if(pos == 0)//thread name
				  return PerftraceUtil.getThreadName(((TRCThread)obj));				  
			}
			
			if(obj instanceof TRCMethodInvocation)
			{
				switch(pos)
				{
					case 0://method name
					   return ((TRCMethodInvocation)obj).getMethod().getName();
					  
					case 1://instance name  
					{
						TRCObject tmpObject = ((TRCMethodInvocation)obj).getOwningObject();
						TRCClass tmpClass = tmpObject.getIsA();
						return tmpClass.getName() + "." + tmpObject.getId();
					}
					
					case 2://base time
					{
						 if(obj instanceof TRCFullMethodInvocation)
						 	return PerftraceUtil.formatTimeValue(((TRCFullMethodInvocation)obj).getEntryTime());
					}
	
					case 3: //cumulative time 
					{
						//exit - entry includes overhead
						
						if(obj instanceof TRCFullMethodInvocation)
						{
							TRCFullMethodInvocation tmpInv = (TRCFullMethodInvocation)obj;
							double start = tmpInv.getEntryTime();
							double end = tmpInv.getExitTime();
							double overhead = tmpInv.getOverhead();
							if(end == 0)
							{
								end = tmpInv.getOwningObject().getProcess().getLastEventTime();
							}
							
							if (_drawMode == TraceConstants.COMPENSATED_TIME)
							{
								return PerftraceUtil.formatTimeValue(end-start-overhead);
							}
							else
							{
								return PerftraceUtil.formatTimeValue(end-start);
							}
							
						}
					}
					
				}
				
			}
							
			return "";
		}
	}
    	
	class ExecutionTreeViewer extends TableTreeViewer {
		public ExecutionTreeViewer(Composite parent) {
			super(parent);
		}

		public ExecutionTreeViewer(TableTree table) {
			super(table);
		}

		public void expandItem(TableTreeItem item) {
			item.setExpanded(true);
			createChildren(item);

		}

	}
	protected Composite createTable(Composite parent, int options) {
		return new TableTree(parent, options);
	}

	public ExecutionStatistic(Composite parent, TraceViewerPage page) {
		super(parent, page);
		
		TraceUIPlugin.getDefault().addTimeChangedEventListener(this);
		_viewerFilter = new ExecutionStatisticFilter();		

	}

	/**
	 * getClassColumnsPerferencesKey returns the string key used to find the
	 * column preferences information in the preferences store.
	 * @return java.lang.String key name
	 */
	public String getColumnsPreferencesKey() {
		// Note this string is not to be translated and must be changed whenever the 
		// default columns template is changed by adding, removing or renaming columns.
		// Changing the version will result in the default preferences being used.
		return "Exec51"; 
		
	}
	
	public String getDefaultColumnsTemplate() {
		//Class Columns Data
		
		String executionColumn =
			    TraceUIPlugin.getString("STR_THREAD_NAME")			+ ":0:"
				+ String.valueOf(ColumnData.NONDELETABLE | ColumnData.IS_VISIBLE | ColumnData.NONMOVABLE) + ":left:200,"
			    + TraceUIPlugin.getString("STR_ST_INSTANCE_INDEX") + ":1:"
			  	+ String.valueOf(ColumnData.IS_VISIBLE) + ":left:200,"					 	
				+ TraceUIPlugin.getString("TITLE_START_TIME") + ":2:" 
					+ String.valueOf(ColumnData.IS_VISIBLE) + ":right:100,"
				+ TraceUIPlugin.getString("TITLE_TIME_ON_STACK") + ":3:"				
					+ String.valueOf(ColumnData.IS_VISIBLE) + ":right:100";
					
		return executionColumn;
	}

/*
	protected StatisticList createStatisticList() {
		return new ThreadStatisticList(this);
	}
*/
	public Table getTable() {
		return ((TableTree) getTableViewer().getControl()).getTable();
		
	}

	protected StructuredViewer createTableViewer(Composite table) {
		ExecutionTreeViewer tv = new ExecutionTreeViewer((TableTree) table);
		return (StructuredViewer) tv;
	}

	protected void handleSelectionEvent() {
	}

	/**
	 * Called when the context menu is about to open.
	 * @see IFillMenuTarget#fillContextMenu
	 */
	public void menuAboutToShow(IMenuManager menu) {
		menu.add(fSeparator);

		menu.add(getUpdateAction());
		
		menu.add(fSeparator);
		menu.add(getChooseColumnsAction(getColumnDataList(), getColumnsPreferencesKey()));
	}


	public void updateButtons() {
	}

	protected void updateDetailsPane() {
		int selIndex = getTable().getSelectionIndex();
		if (selIndex < 0) {
			return;
		}

		Item item = getTable().getItem(selIndex);
		Object itemData = item.getData();

		if (itemData == null) {
			return;
		}
		if (itemData != null && itemData instanceof TableTreeItem) {
			itemData = ((TableTreeItem) itemData).getData();
		}
	}

	public void updateModelSelection() {
		
		ISelection selection = getTableViewer().getSelection();
		if(selection != null && !selection.isEmpty())
		{
			Object sel = ((IStructuredSelection)selection).getFirstElement();
			
			UIPlugin.getDefault().getSelectionModel(
				_page.getMOFObject()).add(
				sel);
	
			ViewSelectionChangedEvent event = UIPlugin.getDefault().getViewSelectionChangedEvent();
			event.setSource(_page.getMOFObject());
			UIPlugin.getDefault().notifyViewSelectionChangedListener(event);			
		}
	}
	
	public void update() {
		if (_firstTime) {
			getTableViewer().addFilter(getViewerFilter());
			_firstTime = false;
			Table table = getTable();
			TableColumn firstColumn = table.getColumn(2);
			
			_viewerSorter = new ExecutionStatisticSorter();

			getViewerSorter().setSortedColumn(firstColumn);
			getTableViewer().setSorter(getViewerSorter());
			
			handleTimeChangedEvent();
		}
		// set the input of the viewer
		getTableViewer().setInput(_page.getMOFObject());

        getTable().setRedraw(false);
		getTableViewer().refresh();
        getTable().setRedraw(true);
        
        handleSelectionEvent();
		
	}
	
  public void handleTimeChangedEvent()
  {		
  	    Action base = ((ExecutionStatisticPage)getTraceViewerPage()).baseTime();
  	    Action raw = ((ExecutionStatisticPage)getTraceViewerPage()).rawTime();
	    _drawMode = TraceUIPlugin.getDefault().getPreferenceStore().getInt(TraceConstants.TIME_OPTION);
	    if(base != null)
	      base.setChecked(_drawMode == TraceConstants.COMPENSATED_TIME);
	    if(raw != null)
	      raw.setChecked(_drawMode == TraceConstants.RAW_TIME);
	
	    String oldName = TraceUIPlugin.getString("TITLE_RAW_TIME_ON_STACK");
		String newName = TraceUIPlugin.getString("TITLE_TIME_ON_STACK");
		if(_drawMode == TraceConstants.RAW_TIME)
		{
		   newName = TraceUIPlugin.getString("TITLE_RAW_TIME_ON_STACK");
		   oldName = TraceUIPlugin.getString("TITLE_TIME_ON_STACK");		   
		}
	
		Table table = getTable();	
		for (int idx = table.getColumnCount(); idx > 0; idx--)
		{
			TableColumn col = table.getColumn(table.getColumnCount() - 1);
			StatisticTableColumnInfo staticTblColInfo = (StatisticTableColumnInfo)(col.getData());
			String columnName = staticTblColInfo.getColumnData().name();	
			if(columnName.equals(oldName) )
			{
			   staticTblColInfo.getColumnData().name(newName);
			   if(staticTblColInfo.isSortColumn())
			   {
			   	   newName = col.getText().substring(0, 1) + newName; 
			   }			   	
 			   col.setText(newName);
			}
		}
	
		getTableViewer().refresh();
  }

	public void dispose()
	{
		super.dispose();
	    TraceUIPlugin.getDefault().removeTimeChangedEventListener(this);
		
	}

	public IContentProvider getContentProvider()
	{
		return new ExecutionStatisticContentProvider();
	}

	public LabelProvider getTableLabelProvider()
	{
		return new ExecutionStatisticLableProvider(this);
	}
	
	public void handleViewSelectionChangedEvent(ViewSelectionChangedEvent event)
	{
		if(!getTraceViewerPage().getTraceViewer().isProcessRefreshEvents()		
		|| getTable().isFocusControl())
		  return;
		  
		handleSelectionEvent();  
	}
	
	public boolean isEmpty()
	{
		if (PerftraceUtil.getAllThreads(_page.getMOFObject()).length > 0)
			return false;
		return true;
	}
	
}

==> hyades.source_3.1.0_20041019_1201.zip: org/eclipse/hyades/trace/views/internal/ExecutionStatistic.java <==
/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.trace.views.internal;

import org.eclipse.hyades.models.trace.TRCClass;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCMethodInvocation;
import org.eclipse.hyades.models.trace.TRCObject;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.trace.ui.TraceViewerPage;
import org.eclipse.hyades.trace.ui.ViewSelectionChangedEvent;
import org.eclipse.hyades.trace.views.actions.internal.OpenSourceAction;
import org.eclipse.hyades.trace.views.adapter.internal.ExecutionStatisticPage;
import org.eclipse.hyades.trace.views.adapter.internal.TraceConstants;
import org.eclipse.hyades.trace.views.internal.CoverageStatistic.CoverageStatisticSorter;
import org.eclipse.hyades.trace.views.internal.PackageStatistic.PackageTreeViewer;
import org.eclipse.hyades.trace.views.internal.StatisticView.StatisticSorter;
import org.eclipse.hyades.trace.views.util.internal.ColumnData;
import org.eclipse.hyades.trace.views.util.internal.ITimeChangedListener;
import org.eclipse.hyades.trace.views.util.internal.PerftraceUtil;
import org.eclipse.hyades.trace.views.util.internal.StatisticTableColumnInfo;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TableTreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.custom.TableTreeItem;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

public class ExecutionStatistic extends StatisticView
	                              implements ITimeChangedListener

{
    protected String tmpString;
    
	private int _drawMode = TraceUIPlugin.getDefault().getPreferenceStore().getInt(TraceConstants.TIME_OPTION);
	
	/**
	 * 
	 *
	 */
	public class ExecutionStatisticFilter extends StatisticFilter {

		public ExecutionStatisticFilter() {
			super();
		}

		public boolean select(Viewer viewer, Object parent, Object element) {
			
			boolean flag = true;
			
			if(!(element instanceof TRCThread))
			  return true;
			  
			String compareText = "";
		   compareText = ((TRCThread)element).getName();
		   if(compareText.equals(""))
			 compareText = TraceUIPlugin.getString("DEFAULT_PACKAGE");
			
			if (_noPattern)
				return true;
			
			if (!_caseSensitive) {
				compareText = compareText.toLowerCase();
			}
			if (_exactMatch) {
				return compareText.compareTo(_prefix) == 0;
			}
			if (_prefix != "") {
				flag = compareText.startsWith(_prefix);
			}
			if (flag && _suffix != "") {
				flag = compareText.endsWith(_suffix);
			}
			if (flag) {
				for (int k = 0; k < _textList.size(); k++) {
					String str1 = (String) _textList.get(k);

					int index = compareText.lastIndexOf(str1);
					if (index == -1) {
						flag = false;
						break;
					}

					compareText = compareText.substring(index + str1.length());
				}
			}
			return flag;
		}
		
	}
	
	public class ExecutionStatisticSorter extends StatisticSorter {

		public ExecutionStatisticSorter() {
			super();
		}

		public int compare(Viewer viewer, Object e1, Object e2) {
			
			double d = 0;
			
			if(e1 instanceof TRCThread && e2 instanceof TRCThread)
			{
				if(_pos == 0)
				  return (int) _sortSequence * (int) PerftraceUtil.getThreadName((TRCThread)e1).compareToIgnoreCase(PerftraceUtil.getThreadName((TRCThread)e2));
			}
			
			if(e1 instanceof TRCFullMethodInvocation && e2 instanceof TRCFullMethodInvocation)
			{
				TRCFullMethodInvocation inv1 = (TRCFullMethodInvocation)e1;
				TRCFullMethodInvocation inv2 = (TRCFullMethodInvocation)e2;
				
				if(_pos == 0) //name
				{
				   return (int) _sortSequence * (int) ((TRCMethodInvocation)e1).getMethod().getName().compareToIgnoreCase(((TRCMethodInvocation)e2).getMethod().getName());
				}
				
				if(_pos == 1) //instances
				{					
				   return (int) _sortSequence * (int) PerftraceUtil.getClass(inv1).getName().compareToIgnoreCase(PerftraceUtil.getClass(inv2).getName());
				}
				
				if(_pos == 2)//base time
				{
					  d = inv1.getEntryTime() - inv2.getEntryTime();
					  
					  if(d < 0)
					     return  -1 * _sortSequence;
					  if(d > 0) 
					    return _sortSequence;
					    
					  return 0;    
				}

				if(_pos == 3) //cumulative time 
				{
					double start = inv1.getEntryTime();
					double end = inv1.getExitTime();
					if(end == 0)
					{
						end = inv1.getOwningObject().getProcess().getLastEventTime();
					}
					d = end-start;
					
					start = inv2.getEntryTime();
					end = inv2.getExitTime();
					if(end == 0)
					{
						end = inv2.getOwningObject().getProcess().getLastEventTime();
					}
					
					  d = d - (end -start);
					  
					  if(d < 0)
					     return  -1 * _sortSequence;
					  if(d > 0) 
					    return _sortSequence;
					    
					  return 0;    
				}						
			}
			
			return 0;
		}
		
	}
	
	/**
	 * 
	 */
	public class ExecutionStatisticContentProvider implements ITreeContentProvider {
		
		public void dispose() {
		}
		
		public Object getParent(Object element) {
			return null;
		}
		
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
		
		public Object[] getElements(Object inputElement)
		{
			return PerftraceUtil.getAllThreads(_page.getMOFObject());
		}
				
		public Object[] getChildren(Object element)
		{
			tmpList.clear();
			
			if(element instanceof TRCThread)
				return ((TRCThread)element).getInitialInvocations().toArray();
			
			TRCMethodInvocation inv = (TRCMethodInvocation)element;
			Object[] segments = inv.getInvokes().toArray();
			for(int idx=0; idx<segments.length; idx++)
			{
				tmpList.add(((TRCMethodInvocation)segments[idx]));
			}
			
			return tmpList.toArray(); 	
		}
	
		public boolean hasChildren(Object element) {
			if (element instanceof TRCThread)
				return true;

			return (((TRCMethodInvocation)element).getInvokes().size() > 0);
		}
	}

    /**
     * 
     */
	public class ExecutionStatisticLableProvider extends LabelProvider implements ITableLabelProvider
	{
		protected StatisticView _viewer;
		
		public ExecutionStatisticLableProvider(StatisticView viewer) {
			super();
			_viewer = viewer;
		}
		
		public Image getColumnImage(Object obj, int col) {
				return null;
		}

		public String getColumnText(Object obj, int col) {
			
            int pos = StatisticTableColumnInfo.getStatisticTableColumnInfo(_viewer.getTable().getColumn(col)).getColumnData().getInitalPos();			
			
			if(obj instanceof TRCThread)	
			{
				if(pos == 0)//thread name
				  return PerftraceUtil.getThreadName(((TRCThread)obj));				  
			}
			
			if(obj instanceof TRCMethodInvocation)
			{
				switch(pos)
				{
					case 0://method name
					   return ((TRCMethodInvocation)obj).getMethod().getName();
					  
					case 1://instance name  
					{
						TRCObject tmpObject = ((TRCMethodInvocation)obj).getOwningObject();
						TRCClass tmpClass = tmpObject.getIsA();
						return tmpClass.getName() + "." + tmpObject.getId();
					}
					
					case 2://base time
					{
						 if(obj instanceof TRCFullMethodInvocation)
						 	return PerftraceUtil.formatTimeValue(((TRCFullMethodInvocation)obj).getEntryTime());
					}
	
					case 3: //cumulative time 
					{
						//exit - entry includes overhead
						
						if(obj instanceof TRCFullMethodInvocation)
						{
							TRCFullMethodInvocation tmpInv = (TRCFullMethodInvocation)obj;
							double start = tmpInv.getEntryTime();
							double end = tmpInv.getExitTime();
							double overhead = tmpInv.getOverhead();
							if(end == 0)
							{
								end = tmpInv.getOwningObject().getProcess().getLastEventTime();
							}
							
							if (_drawMode == TraceConstants.COMPENSATED_TIME)
							{
								return PerftraceUtil.formatTimeValue(end-start-overhead);
							}
							else
							{
								return PerftraceUtil.formatTimeValue(end-start);
							}
							
						}
					}
					
				}
				
			}
							
			return "";
		}
	}
    	
	class ExecutionTreeViewer extends TableTreeViewer {
		public ExecutionTreeViewer(Composite parent) {
			super(parent);
		}

		public ExecutionTreeViewer(TableTree table) {
			super(table);
		}

		public void expandItem(TableTreeItem item) {
			item.setExpanded(true);
			createChildren(item);

		}

	}
	protected Composite createTable(Composite parent, int options) {
		return new TableTree(parent, options);
	}

	public ExecutionStatistic(Composite parent, TraceViewerPage page) {
		super(parent, page);
		
		TraceUIPlugin.getDefault().addTimeChangedEventListener(this);
		_viewerFilter = new ExecutionStatisticFilter();		

	}

	/**
	 * getClassColumnsPerferencesKey returns the string key used to find the
	 * column preferences information in the preferences store.
	 * @return java.lang.String key name
	 */
	public String getColumnsPreferencesKey() {
		// Note this string is not to be translated and must be changed whenever the 
		// default columns template is changed by adding, removing or renaming columns.
		// Changing the version will result in the default preferences being used.
		return "Exec51"; 
		
	}
	
	public String getDefaultColumnsTemplate() {
		//Class Columns Data
		
		String executionColumn =
			    TraceUIPlugin.getString("STR_THREAD_NAME")			+ ":0:"
				+ String.valueOf(ColumnData.NONDELETABLE | ColumnData.IS_VISIBLE | ColumnData.NONMOVABLE) + ":left:200,"
			    + TraceUIPlugin.getString("STR_ST_INSTANCE_INDEX") + ":1:"
			  	+ String.valueOf(ColumnData.IS_VISIBLE) + ":left:200,"					 	
				+ TraceUIPlugin.getString("TITLE_START_TIME") + ":2:" 
					+ String.valueOf(ColumnData.IS_VISIBLE) + ":right:100,"
				+ TraceUIPlugin.getString("TITLE_TIME_ON_STACK") + ":3:"				
					+ String.valueOf(ColumnData.IS_VISIBLE) + ":right:100";
					
		return executionColumn;
	}

/*
	protected StatisticList createStatisticList() {
		return new ThreadStatisticList(this);
	}
*/
	public Table getTable() {
		return ((TableTree) getTableViewer().getControl()).getTable();
		
	}

	protected TableTreeViewer getTableTreeViewer(TableTree table)
	{
		return new ExecutionTreeViewer(table);
	}

	protected void handleSelectionEvent() {
	}

	/**
	 * Called when the context menu is about to open.
	 * @see IFillMenuTarget#fillContextMenu
	 */
	public void menuAboutToShow(IMenuManager menu) {
		menu.add(fSeparator);

		menu.add(getUpdateAction());
		
		menu.add(fSeparator);
		menu.add(getChooseColumnsAction(getColumnDataList(), getColumnsPreferencesKey()));
	}


	public void updateButtons() {
	}

	protected void updateDetailsPane() {
		int selIndex = getTable().getSelectionIndex();
		if (selIndex < 0) {
			return;
		}

		Item item = getTable().getItem(selIndex);
		Object itemData = item.getData();

		if (itemData == null) {
			return;
		}
		if (itemData != null && itemData instanceof TableTreeItem) {
			itemData = ((TableTreeItem) itemData).getData();
		}
	}

	public void updateModelSelection() {
		
		ISelection selection = getTableViewer().getSelection();
		if(selection != null && !selection.isEmpty())
		{
			Object sel = ((IStructuredSelection)selection).getFirstElement();
			
			notifyViewSelectionChanged(this,sel);
		}
	}
	
	protected StatisticSorter getViewerSorterInstance()
	{
		return new ExecutionStatisticSorter();
	}
	
	protected void firstTimeEvents()
	{
		handleTimeChangedEvent();		
	}	
	
	protected int updateTableGetColumnNumber()
	{
		return 2;
	}	
	
  public void handleTimeChangedEvent()
  {		
  	    Action base = ((ExecutionStatisticPage)getTraceViewerPage()).baseTime();
  	    Action raw = ((ExecutionStatisticPage)getTraceViewerPage()).rawTime();
	    _drawMode = TraceUIPlugin.getDefault().getPreferenceStore().getInt(TraceConstants.TIME_OPTION);
	    if(base != null)
	      base.setChecked(_drawMode == TraceConstants.COMPENSATED_TIME);
	    if(raw != null)
	      raw.setChecked(_drawMode == TraceConstants.RAW_TIME);
	
	    String oldName = TraceUIPlugin.getString("TITLE_RAW_TIME_ON_STACK");
		String newName = TraceUIPlugin.getString("TITLE_TIME_ON_STACK");
		if(_drawMode == TraceConstants.RAW_TIME)
		{
		   newName = TraceUIPlugin.getString("TITLE_RAW_TIME_ON_STACK");
		   oldName = TraceUIPlugin.getString("TITLE_TIME_ON_STACK");		   
		}
	
		Table table = getTable();	
		for (int idx = table.getColumnCount(); idx > 0; idx--)
		{
			TableColumn col = table.getColumn(table.getColumnCount() - 1);
			StatisticTableColumnInfo staticTblColInfo = (StatisticTableColumnInfo)(col.getData());
			String columnName = staticTblColInfo.getColumnData().name();	
			if(columnName.equals(oldName) )
			{
			   staticTblColInfo.getColumnData().name(newName);
			   if(staticTblColInfo.isSortColumn())
			   {
			   	   newName = col.getText().substring(0, 1) + newName; 
			   }			   	
 			   col.setText(newName);
			}
		}
	
		getTableViewer().refresh();
  }

	public void dispose()
	{
		super.dispose();
	    TraceUIPlugin.getDefault().removeTimeChangedEventListener(this);
		
	}

	public IContentProvider getContentProvider()
	{
		return new ExecutionStatisticContentProvider();
	}

	public LabelProvider getTableLabelProvider()
	{
		return new ExecutionStatisticLableProvider(this);
	}
	
	public void handleViewSelectionChangedEvent(ViewSelectionChangedEvent event)
	{
		Object  source = event.getSource();
		if(source!=this){	  
			handleSelectionEvent();
		}
	}
	
	public boolean isEmpty()
	{
		if (PerftraceUtil.getAllThreads(_page.getMOFObject()).length > 0)
			return false;
		return true;
	}
	
    protected String getViewTypeStr()
    {
    	return org.eclipse.hyades.trace.views.adapter.internal.TraceConstants.EXECUTION_STATS_VIEW;
    }			
}
